
' EAST YORKSHIRE COASTAL EROSION CALCULATOR
' Calculates cliff loss at single location over a range of years
' Development by Brian Williams
' Liberty BASIC Pro v4.04 program
' erosion-calculator v2 April 2014
' Last change 14 September 2022

' Data obtained from East Riding of Yorkshire Council
' 27/09/18: Input of data from Excel file "data-only-calculator.csv"
'   data-only-calculator.csv differs from data-only.csv by the expansion of all seasonal data
'   i.e. cumulative values (following a gap in surveys) are divided out across columns
' 17/11/20: data-only-calculator.csv: Prs 58 to 70 ERYC data replaced by own measurements for the period 1852/1854 to 1952/1956
 

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

' PRESS SHIFT + F5 TO RUN PROGRAM

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

'   Date is read into an array as follow:

'   Rows 1-123           data from erosion monitoring profiles 1-123
'   Row 124              nominal date of survey (for program)
'   Row 125              tabled date of survey (ERYC)
'
'   DF                   defended (seaside) frontage
'   RS                   records started
'
'   COLUMNS WILL INCREASE IN NUMBER AS NEW (LATEST) DATA IS ADDED - adjustment is automatic
'
'   First and penultimate columns contain monitoring profile reference numbers
'   Final column gives location of measuring profiles
'
'   Column/nominal year  2: 1853  3: 1889  4: 1907  5: 1927
'                        6: 1954  7: 1969  8: 1975  9: 1977 10: 1989  11: 1992  12: 1994  13: 1995  14: 1996  15: 1997
'
'                        (In spring 1999, survey methodology changed from calculations using OS maps to GPS technology, itself later superseded by LiDAR.
'                         At the same time, twice yearly monitoring was introduced, made regular by 2003 at all locations.)
'
'                        16: 1998         17: 1999 spring - merged by the program into year 1998
'                        18: 1999 autumn  19: 2000 spring - merged by the program into year 1999
'                        20: 2000 autumn  21: 2001 spring - merged by the program into year 2000
'                        22: 2001 autumn  23: 2002 spring - merged by the program into year 2001
'                        24: 2002 autumn  25: 7003 spring - merged by the program into year 2002
'                        26: 2003 autumn  27: 9004 spring - merged by the program into year 2003
'                        28: 2004 autumn  29: 2005 spring - merged by the program into year 2004
'                        30: 2005 autumn  31: 2006 spring - merged by the program into year 2005
'                        32: 2006 autumn  33: 2007 spring - merged by the program into year 2006
'                        34: 2007 autumn  35: 2008 spring - merged by the program into year 2007
'                        36: 2008 autumn  37: 2009 spring - merged by the program into year 2008
'                        38: 2009 autumn  39: 2010 spring - merged by the program into year 2009
'                        40: 2010 autumn  41: 2011 spring - merged by the program into year 2010
'                        42: 2011 autumn  43: 2012 spring - merged by the program into year 2011
'                        44: 2012 autumn  45: 2013 spring - merged by the program into year 2012
'                        46: 2013 autumn  47: 2014 spring - merged by the program into year 2013 (combined December 2013 surge and May 2014 surveys)
'                        48: 2014 autumn  49: 2015 spring - merged by the program into year 2014
'                        50: 2015 autumn  51: 2016 spring - merged by the program into year 2015
'                        52: 2016 autumn  53: 2017 spring - merged by the program into year 2016 (no survey carried out for autumn 2016)
'                        54: 2017 autumn  55: 2018 spring - merged by the program into year 2017
'                        56: 2018 autumn  57: 2019 spring - merged by the program into year 2018
'                        58: 2019 autumn  59: 2020 spring - merged by the program into year 2019
'                        60: 2020 autumn  61: 2021 spring - merged by the program into year 2020
'                        62: 2021 autumn  63: 2022 spring - merged by the program into year 2021
'
'   Array column: 1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26  
'   Excel column: A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z 
'
'                 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 
'                 AA AB AC AD AE AF AG AH AI AJ AK AL AM AN AO AP AQ AR AS AT AU AV AW AX AY AZ
'
'                 53 54 55 56 57 58 59 60 61 62 63
'                 BA BB BC BD BE BF BG BH BI BJ BK
'   merged columns                                             ----  ----  ----  ----  ---- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- ----- -----
'
'   > > > Latest data situation set manually at first two lines of program, and in "Last update ..." in program header text
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  - - -

       LatestDataYear=2021                                                                              ' array dimensions adjusted by program
       HalfYearFlag=0                                                                                   ' 0=full year (autumn and spring), 1=half year (autumn only)
                                                                                                        ' * * * half year option no longer used * * *

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  - - -

       ColCalc=15+(2*(LatestDataYear-1997))+2                                                           ' adjusts variables on addition of new data

       dim Array$(125,ColCalc)
       dim Array (125,ColCalc)

       print
       print " EAST YORKSHIRE COASTAL EROSION CALCULATOR"
       print " Calculates cliff loss at a selected location over a range of years"
       print
       print " Prepared by Brian Williams in 2012"
       print " Last update September 2022"                                                              ' update information
       print " Data obtained from East Riding of Yorkshire Council"
       print " Every care taken but no liability implied"
       print " Future cliff losses cannot be assumed from past rates of erosion"
       print
       print " This program merges data collected at half-yearly intervals into a nominal year"
       print "  (applies from 1998)"
       print
       print " www.urbanrim.org.uk/erosion-calculator"
       print " ___________________________________________________________________________"




'   fill input array from Excel file

      open "data-only-calculator.csv" for input as #1
        while eof(#1)=0
          for row=1 to 125
            for col=1 to ColCalc
              input #1, Array$(row,col)
                select case Array$(row,col)
                  case "RS"                                                                             ' "Records Started" given numerical value 999
                    Array$(row,col)="999"
                  case "DF"                                                                             ' "Defended Frontage" given numerical value 9999
                    Array$(row,col)="9999"
                  case ""                                                                               ' "no record taken" denoted by -1
                    Array$(row,col)="-1"
                end select
                if row<125 and col<(ColCalc-1) then Array(row,col)=val(Array$(row,col))                 ' converts data input string to numerical value
            next col
          next row
        wend
      close #1




'   merge half years to nominal year

      for FindHalfYears=2 to (ColCalc-2)
        if int(Array(124,FindHalfYears))=Array(124,FindHalfYears) then [NextFind]
          for MergeHalfYears=1 to 123
            if (Array(MergeHalfYears,FindHalfYears)=-1) or (Array(MergeHalfYears,FindHalfYears)=999) or (Array(MergeHalfYears,FindHalfYears)=9999) then [NextMerge]
            if Array(MergeHalfYears,FindHalfYears+1)=-1 then Array(MergeHalfYears,FindHalfYears+1)=0    ' deals with "no record taken" value
            Array(MergeHalfYears,FindHalfYears+1)=Array(MergeHalfYears,FindHalfYears+1)+Array(MergeHalfYears,FindHalfYears)
           Array(MergeHalfYears,FindHalfYears)=-1
[NextMerge]
          next MergeHalfYears
[NextFind]
      next FindHalfYears




'   menu

[Menu]
      n$=""
      print:print
      print " calculator year range is 1854 to "; LatestDataYear
      print
      print " 'n' and <Enter> to end program"
      print " 'l' and <Enter> for locations list"
      input "         <Enter> to continue       "; n$
        if n$="n" then print:print:goto [End]
        if n$="l" then print:print:for Location=1 to 123:print "    "; using("###", val(Array$(Location,ColCalc-1))); " "; Array$(Location,ColCalc):next Location:goto [Menu]
        if n$<>"" then goto [Menu]




'   user input (location and year)

[Location]
      print:print
      input " enter location number:  "; PostRow
        if (PostRow <1) or (PostRow>123) then print " locations are numbered from 1 to 123":print:print:goto [Location]           ' range exceeded
      print "                         "; Array$(PostRow,ColCalc)
        if Array(PostRow,2)=9999 then print " defended frontage - no erosion":print:print:goto [Location]                         ' defended frontages

[FirstYear]
      input " enter first year of required period:    "; StartYear

        if StartYear>LatestDataYear then print " data ends at "; LatestDataYear:print:print:goto [FirstYear]

        if PostRow=1 and StartYear<2003 then print " monitoring at this location started 2003":print:print:goto [FirstYear]       ' special case
        if PostRow=2 and StartYear<2003 then print " monitoring at this location started 2003":print:print:goto [FirstYear]       ' special case
        if PostRow=12 and StartYear<1908 then print " monitoring at this location started 1908":print:print:goto [FirstYear]      ' special case
        if PostRow=13 and StartYear<1908 then print " monitoring at this location started 1908":print:print:goto [FirstYear]      ' special case

        if StartYear<1854 then print " data starts at 1854":print:print:goto [FirstYear]

[LastYear]
      input " enter last year of required period:     "; EndYear:print:print
        if EndYear<1854+1 then print " the last year has to be at least 1855": goto [LastYear]
        if EndYear>LatestDataYear then print " the last year has to be no later than "; LatestDataYear:goto [LastYear]
        if EndYear<StartYear then print " end year cannot precede start year":print:print:goto [LastYear]

        if PostRow=94 and EndYear>2019 then print " protected by extented defences from 2020":print:print:goto [LastYear]         ' special case
        if PostRow=110 and EndYear>1999 then print " protected by defences from 1999":print:print:goto [LastYear]                 ' special case
        if PostRow=111 and EndYear>1998 then print " protected by defences from 1998":print:print:goto [LastYear]                 ' special case
        if PostRow=112 and EndYear>1998 then print " protected by defences from 1998":print:print:goto [LastYear]                 ' special case
        if PostRow=115 and EndYear>1994 then print " cliff line lost within lagoon site in 1994":print:print:goto [LastYear]      ' special case
        if PostRow=116 and EndYear>1994 then print " cliff line lost within lagoon site in 1994":print:print:goto [LastYear]      ' special case
        if PostRow=117 and EndYear>1994 then print " cliff line lost within lagoon site in 1994":print:print:goto [LastYear]      ' special case
        if PostRow=123 and EndYear>2013 then print " position lost during winter 2013":print:print:goto [LastYear]                ' special case
        
      print " ---------------------------------":print




'   loop

      for Year=StartYear to EndYear
        DataCol=2




'   data retrieval

        while Array(124,DataCol)<Year                                                                   ' finds the data year that follows the user's enquiry year
          DataCol=DataCol+1
        wend

          if Array(PostRow,DataCol)>-1 then goto [DataEnd]                                              ' data year contains a record for the location
            while Array(PostRow,DataCol)=-1                                                             ' finds a later record
              DataCol=DataCol+1
            wend
[DataEnd]
          DataSpanEnd=Array(124,DataCol):DataEnd=Array(PostRow,DataCol)                                 ' sets the end date of a survey period

            if Array(PostRow,(DataCol-1))>-1 then goto [DataStart]                                      ' previous data year contains a record for the location
              while Array(PostRow,DataCol-1)=-1                                                         ' finds a previous record or RS
                DataCol=DataCol-1
              wend
[DataStart]
          DataSpanStart=Array(124,DataCol-1)                                                            ' sets the start date of a survey period




'   calculation

          DataSpan=DataSpanEnd-DataSpanStart                                                            ' length of survey (in years)
          SpanAvg=DataEnd/DataSpan                                                                      ' erosion rate


'   results (output)

          space$="":pluralmetres$="s":pluralyears$="s"
            if SpanAvg>10 then space$=" "
            if SpanAvg=1.00 then pluralmetres$=""
            if DataSpan=1 then pluralyears$=""
            if DataSpanStart=-1 then goto [NextYear]                                                    ' special cases - profiles 1, 2, 12, 13 late start to monitoring
            if HalfYearFlag=1 and Year=LatestDataYear then DataSpan=0.5: HalfYear$="half "

          print " Year                         "; Year
          print " survey period starts         "; DataSpanStart
          print " survey period ends           "; DataSpanEnd
          print " length of survey period      "; DataSpan; " year"; pluralyears$
            if DataSpan=0.5 then print " (data for second half of "; LatestDataYear;" not yet available)"
          print " erosion rate:               "; space$; using("##.##", SpanAvg); " metre"; pluralmetres$; " per "; HalfYear$; "year"
          print:print

          LossCount=LossCount+SpanAvg
          YearCount=YearCount+1

          DataSpanStart=0: DataSpanEnd=0: DataSpan=0: SpanAvg=0
          DataEnd=0
          HalfYear$=""

[NextYear]
      next Year




'   summary

      print " ---------------------------------":print
        if HalfYearFlag=1 and EndYear=LatestDataYear then YearCount=YearCount-0.5                       ' adjusts for half year
      AverageErosion=LossCount/YearCount

      print " SUMMARY"
      print " location "; PostRow; "  "; Array$(PostRow,ColCalc)
        if StartYear=EndYear then print " years displayed              "; "1 ("; EndYear;")":goto [LossCount] ' first and last year the same
      print " years displayed              "; YearCount; " ("; StartYear; " to "; EndYear;")"

[LossCount]
        select case
          case (LossCount <10)
            space$=""
          case (LossCount >=10 and LossCount <100)
            space$=" "
          case (LossCount >=100)
            space$="  "
        end select
      print " total cliff loss           "; space$; using ("###.##", LossCount); " metre"; pluralmetres$ 
        if AverageErosion<10 then space$="" else space$=" "
      print " average erosion             "; space$; using ("##.##", AverageErosion); " metre"; pluralmetres$; " per year"
      print:print " ---------------------------------":print
      LossCount=0:YearCount=0:MDtrip=0

      goto [Menu]




'   end

[End] print " ___________________________________________________________________________"
      print:print " Program ends"
      end





